package com.cpp.web.service.accuracy;

import com.cpp.common.utils.spring.SpringUtils;
import com.cpp.system.service.ISysConfigService;
import com.cpp.web.domain.accuracy.AccuracyPassRate;
import com.cpp.web.domain.station.ElectricField;
import com.cpp.web.domain.station.PowerStationStatusData;
import com.cpp.web.domain.station.enums.ElectricFieldTypeEnum;
import com.cpp.web.service.cloud.ForecastPowerShortTermCloudService;
import com.cpp.web.service.cloud.ForecastPowerUltraShortTermCloudService;
import com.cpp.web.service.station.*;
import com.cpp.web.utils.DateTimeUtil;
import com.syjy.calculate.entity.CalculateRequest;
import com.syjy.calculate.entity.CalculateResult;
import com.syjy.calculate.entity.CalculationFormula;
import com.syjy.calculate.entity.CalculationInfo;
import com.syjy.calculate.repository.CalculationFormulaRepository;
import com.syjy.calculate.service.AccuracyPassRateCalculateService;
import lombok.RequiredArgsConstructor;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


@Service
@RequiredArgsConstructor
public class CalcAccuracy {

    public final AccuracyPassRateService accuracyPassRateService;
    private final ElectricFieldService electricFieldService;
    private final ISysConfigService sysConfigService;
    private final PowerStationStatusDataService powerStationStatusDataService;
    private final List<CalculateInterface> calculateInterfaces;

    /**
     * 计算方法
     */
//    @Scheduled(cron = "0 51 1 * * *")
//    @Scheduled(fixedRate = 60000L)
    public void calculate() {
        Long day = 86400000L;
        Long now = System.currentTimeMillis();

        Date startTime = DateTimeUtil.getDayStartTime(now - day);
        Date endTime = DateTimeUtil.getDayLastTime(now - day);
        List<ElectricField> electricFieldList = electricFieldService.list();
        List<AccuracyPassRate> accuracyPassRateList = new ArrayList<>();
        String province = sysConfigService.selectConfigByKey("province");
        if (province.equals("")) province = "E65";
        if (!electricFieldList.isEmpty()) {
            List<PowerStationStatusData> powerStationStatusDataList = powerStationStatusDataService.findByTimeBetween(startTime, endTime);
            if (!powerStationStatusDataList.isEmpty()) {
                List<String> formulaTypes = Arrays.asList("DAY_SHORT_ACCURACY", "POINT_SHORT_ACCURACY", "DAY_ULTRA_SHORT_ACCURACY");
                for (CalculateInterface calculateInterface : calculateInterfaces) {
                    if (listHasIntersection(calculateInterface.calculationTypes(), formulaTypes)) {
                        accuracyPassRateList.addAll(calculateInterface.calc(startTime, endTime, powerStationStatusDataList, electricFieldList, formulaTypes, province));
                    }
                }
            }
        }
        if (accuracyPassRateList.size() > 0) {
            accuracyPassRateService.saveBatch(accuracyPassRateList);
        }
    }

    public Boolean listHasIntersection(List<String> strings, List<String> stringList) {
        for (String string : strings) {
            for (String s : stringList) {
                if (string.equals(s)) {
                    return true;
                }
            }
        }
        return false;
    }
}
